@@ -1,6 +1,6 @@
 # -*- test-case-name: idavoll.test.test_gateway -*-
 #
-# Copyright (c) 2003-2008 Ralph Meijer
+# Copyright (c) 2003-2009 Ralph Meijer
 # See LICENSE for details.
 
 """
@@ -68,7 +68,7 @@ def getServiceAndNode(uri):
     try:
         service = JID(entity)
     except Exception, e:
-        raise XMPPURIParseError("Invalid JID: %s" % e.message)
+        raise XMPPURIParseError("Invalid JID: %s" % e)
 
     params = cgi.parse_qs(query)
 
@@ -171,10 +171,7 @@ class DeleteResource(resource.Resource):
         Respond to a POST request to create a new node.
         """
 
-        def respond(result):
-            return http.Response(responsecode.NO_CONTENT)
-
-        def getNode():
+        def gotStream(_):
             if request.args.get('uri'):
                 jid, nodeIdentifier = getServiceAndNode(request.args['uri'][0])
                 return defer.succeed(nodeIdentifier)
@@ -182,6 +179,20 @@ class DeleteResource(resource.Resource):
                 raise http.HTTPError(http.Response(responsecode.BAD_REQUEST,
                                                    "No URI given"))
 
+        def doDelete(nodeIdentifier, data):
+            if data:
+                params = simplejson.loads(''.join(data))
+                redirectURI = params.get('redirect_uri')
+            else:
+                redirectURI = None
+
+            return self.backend.deleteNode(nodeIdentifier, self.owner,
+                                           redirectURI)
+
+        def respond(result):
+            return http.Response(responsecode.NO_CONTENT)
+
+
         def trapNotFound(failure):
             failure.trap(error.NodeNotFound)
             return http.StatusResponse(responsecode.NOT_FOUND,
@@ -190,10 +201,12 @@ class DeleteResource(resource.Resource):
         def trapXMPPURIParseError(failure):
             failure.trap(XMPPURIParseError)
             return http.StatusResponse(responsecode.BAD_REQUEST,
-                    "Malformed XMPP URI: %s" % failure.value.message)
+                    "Malformed XMPP URI: %s" % failure.value)
 
-        d = getNode()
-        d.addCallback(self.backend.deleteNode, self.owner)
+        data = []
+        d = readStream(request.stream, data.append)
+        d.addCallback(gotStream)
+        d.addCallback(doDelete, data)
         d.addCallback(respond)
         d.addErrback(trapNotFound)
         d.addErrback(trapXMPPURIParseError)
@@ -278,7 +291,7 @@ class PublishResource(resource.Resource):
         def trapXMPPURIParseError(failure):
             failure.trap(XMPPURIParseError)
             return http.StatusResponse(responsecode.BAD_REQUEST,
-                    "Malformed XMPP URI: %s" % failure.value.message)
+                    "Malformed XMPP URI: %s" % failure.value)
 
         self.checkMediaType(request)
         d = self.parseXMLPayload(request.stream)
@@ -473,11 +486,14 @@ class RemoteSubscriptionService(service.Service, PubSubClient):
 
         service = event.sender
         nodeIdentifier = event.nodeIdentifier
-        self.callCallbacks(service, nodeIdentifier, eventType='DELETED')
+        redirectURI = event.redirectURI
+        self.callCallbacks(service, nodeIdentifier, eventType='DELETED',
+                           redirectURI=redirectURI)
 
 
     def _postTo(self, callbacks, service, nodeIdentifier,
-                      payload=None, contentType=None, eventType=None):
+                      payload=None, contentType=None, eventType=None,
+                      redirectURI=None):
 
         if not callbacks:
             return
@@ -495,6 +511,11 @@ class RemoteSubscriptionService(service.Service, PubSubClient):
         if eventType:
             headers['Event'] = eventType
 
+        if redirectURI:
+            headers['Link'] = '<%s>; rel=alternate' % (
+                              redirectURI.encode('utf-8'),
+                              )
+
         def postNotification(callbackURI):
             d = client.getPage(str(callbackURI),
                                    method='POST',
@@ -507,7 +528,8 @@ class RemoteSubscriptionService(service.Service, PubSubClient):
 
 
     def callCallbacks(self, service, nodeIdentifier,
-                            payload=None, contentType=None, eventType=None):
+                            payload=None, contentType=None, eventType=None,
+                            redirectURI=None):
 
         def eb(failure):
             failure.trap(error.NoCallbacks)
@@ -516,7 +538,7 @@ class RemoteSubscriptionService(service.Service, PubSubClient):
 
         d = self.storage.getCallbacks(service, nodeIdentifier)
         d.addCallback(self._postTo, service, nodeIdentifier, payload,
-                                    contentType, eventType)
+                                    contentType, eventType, redirectURI)
         d.addErrback(eb)
         d.addErrback(log.err)
 
@@ -579,7 +601,7 @@ class RemoteSubscribeBaseResource(resource.Resource):
         def trapXMPPURIParseError(failure):
             failure.trap(XMPPURIParseError)
             return http.StatusResponse(responsecode.BAD_REQUEST,
-                    "Malformed XMPP URI: %s" % failure.value.message)
+                    "Malformed XMPP URI: %s" % failure.value)
 
         d = readStream(request.stream, storeParams)
         d.addCallback(gotRequest)
@@ -708,7 +730,10 @@ class CallbackResource(resource.Resource):
 
     def http_POST(self, request):
         p = WebStreamParser()
-        d = p.parse(request.stream)
+        if not request.headers.hasHeader('Event'):
+            d = p.parse(request.stream)
+        else:
+            d = defer.succeed(None)
         d.addCallback(self.callback, request.headers)
         d.addCallback(lambda _: http.Response(responsecode.NO_CONTENT))
         return d
@@ -769,6 +794,25 @@ class GatewayClient(service.Service):
         return f.deferred.addCallback(simplejson.loads)
 
 
+    def delete(self, xmppURI, redirectURI=None):
+        query = {'uri': xmppURI}
+
+        if redirectURI:
+            params = {'redirect_uri': redirectURI}
+            postdata = simplejson.dumps(params)
+            headers = {'Content-Type': MIME_JSON}
+        else:
+            postdata = None
+            headers = None
+
+        f = getPageWithFactory(self._makeURI('delete', query),
+                    method='POST',
+                    postdata=postdata,
+                    headers=headers,
+                    agent=self.agent)
+        return f.deferred
+
+
     def publish(self, entry, xmppURI=None):
         query = xmppURI and {'uri': xmppURI}
 
